package ch.jaros.labyrinth;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Random;

/**
 * @author Martin Jaros
 */
public class Finder {
	private Labyrinth labyrinth;

	private List notCompletedPoints = new LinkedList();
	private Random random = new Random();

	public Finder(Labyrinth labyrinth) {
		this.labyrinth = labyrinth;
	}

	public Point nextJunction() {
		Point nextJunction = null;
		Iterator it = notCompletedPoints.iterator();

		while (it.hasNext() && (nextJunction == null)) {
			Point notCompletedPoint = (Point) it.next();
			List freePointsAround = getFreePoints(notCompletedPoint.getPosition());
			
			int numberOfFreePoints = freePointsAround.size();
			
			if (numberOfFreePoints > 0) {
				nextJunction = notCompletedPoint;
			}
			
			if (numberOfFreePoints <= 1) {
				it.remove();
			}
		}

		return nextJunction;
	}

	public Point nextPoint() {
		Point nextPoint = null;
		List possiblePoints = getFreePoints(labyrinth.getPoint().getPosition());
		int numberOfPoints = possiblePoints.size();

		if (numberOfPoints > 0) {
			nextPoint =
				(Point) possiblePoints.get(random.nextInt(numberOfPoints));
		}

		if (numberOfPoints > 1) {
			notCompletedPoints.add(labyrinth.getPoint());
		}

		return nextPoint;
	}

	private List getFreePoints(Position position) {
		List freePoints = new ArrayList();
		
		Iterator it = position.getPositionsAround().iterator();

		while (it.hasNext()) {
			Position positionAround = (Position) it.next();
			Point point = labyrinth.getPoint(positionAround);

			if (!point.isTaken()) {
				freePoints.add(point);
			}
		}

		return freePoints;
	}
}