package ch.jaros.labyrinth;

import java.util.Random;

/**
 * @author Martin Jaros
 */
public abstract class Labyrinth {
	
	protected Connector connector;
	private Point[][] field;
	private int width;
	private int height;
	private Position currentPosition;
	private Random random = new Random();


	public Labyrinth(int width, int height) {
		field = new Point[width + 2][height + 2];
		this.width = width;
		this.height = height;
		connector = new Connector(this);
	}
	
	public void generate() {
		prepareField();

		connector.generateConnections();

		makeEntries();
		
		connector = null;
	}

	protected void prepareField() {
		for (int y = 0; y <= height + 1; y++) {
			for (int x = 0; x <= width + 1; x++) {
				Position position = getPositionInstance(x, y);
				Point point = new Point(position);
				field[x][y] = point;

				if ((x == 0)
					|| (y == 0)
					|| (x == (width + 1) || (y == height + 1))) {
					point.setTaken(true);
				}
			}
		}
	
		currentPosition =
		getPositionInstance(random.nextInt(width) + 1, random.nextInt(height) + 1);
		
		getPoint(currentPosition).setTaken(true);
	}

	protected void makeEntries() {
		Position position11 = getPositionInstance(random.nextInt(width) + 1, 0);
		Position position12 = getPositionInstance(position11.getX(), position11.getY() + 1);
		Position position21 = getPositionInstance(random.nextInt(width) + 1, height + 1);
		Position position22 = getPositionInstance(position21.getX(), position21.getY() - 1);
		
		Point point11 = getPoint(position11);
		Point point12 = getPoint(position12);
		connector.connectPoints(point11, point12);
		
		Point point21 = getPoint(position21);
		Point point22 = getPoint(position22);
		connector.connectPoints(point21, point22);
	}

	public Point getPoint(Position position) {
		return field[position.getX()][position.getY()];
	}

	public Point getPoint() {
		return getPoint(currentPosition);
	}

	public int getHeight() {
		return height;
	}

	public int getWidth() {
		return width;
	}

	public void setCurrentPosition(Position position) {
		currentPosition = position;
	}

	abstract protected Position getPositionInstance(int x, int y);
}