/*
 * Created on Feb 23, 2004
 */
package ch.jaros.labyrinth.application;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.GraphicsEnvironment;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JScrollPane;

import ch.jaros.labyrinth.graphic.LabyrinthOutputControls;
import ch.jaros.labyrinth.graphic.OutputGenerator;

/**
 * @author Martin Jaros
 */
public class OutputWindow extends JFrame {

	private static int WINDOW_WIDTH = 500;
	private static int WINDOW_HEIGHT = 200;
	private static int WINDOW_HEIGHT_MAX =
		GraphicsEnvironment
			.getLocalGraphicsEnvironment()
			.getScreenDevices()[0]
			.getDisplayMode()
			.getHeight()
			- 30;
	private static int WINDOW_WIDTH_MAX =
		GraphicsEnvironment
			.getLocalGraphicsEnvironment()
			.getScreenDevices()[0]
			.getDisplayMode()
			.getWidth();

	private JPanel mainPanel = new JPanel(new BorderLayout());
	private JPanel controlPanel = new JPanel(new GridBagLayout());
	private JButton windowSizeButton = new JButton("Fenstergrsse anpassen");
	private JButton closeButton = new JButton("Schliessen");

	private JPanel outputPanel = new JPanel(new BorderLayout());
	private JScrollPane scroll = new JScrollPane();

	public OutputWindow(final LabyrinthOutputControls labyrinthOutputControls) {
		super("Labyrinth-Generator Output");

		this.getContentPane().add(mainPanel);

		mainPanel.add(outputPanel, BorderLayout.CENTER);
		mainPanel.add(controlPanel, BorderLayout.SOUTH);

		scroll.getViewport().add(
			labyrinthOutputControls.getLabyrinthComponent());

		outputPanel.add(scroll, BorderLayout.CENTER);

		GridBagConstraints constraints = new GridBagConstraints();
		constraints.fill = GridBagConstraints.HORIZONTAL;
		constraints.gridx = 0;
		constraints.gridy = 0;
		constraints.insets = new Insets(2, 2, 2, 2);
		constraints.weightx = 1;
		controlPanel.add(new JPanel(), constraints);
		constraints.weightx = 0;
		constraints.gridx++;
		controlPanel.add(labyrinthOutputControls.getCellPanel(), constraints);
		constraints.gridx++;
		controlPanel.add(labyrinthOutputControls.getPrintButton(), constraints);
		constraints.gridx++;
		controlPanel.add(windowSizeButton, constraints);
		constraints.gridx++;
		controlPanel.add(closeButton, constraints);

		calculateSize();

		windowSizeButton.addActionListener(new ActionListener() {

			public void actionPerformed(ActionEvent e) {
				OutputWindow.this.calculateSize();
			}
		});

		closeButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				OutputWindow.this.dispose();
			}
		});

		labyrinthOutputControls
			.getPrintButton()
			.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent arg0) {
					OutputGenerator.print(labyrinthOutputControls.getLabyrinthComponent());
				}
		});

		this.addWindowListener(new WindowAdapter() {
			public void windowClosing(WindowEvent arg0) {
				OutputWindow.this.dispose();
			}
		});

		this.addComponentListener(new ComponentAdapter() {
			public void componentResized(ComponentEvent e) {
				Component component = e.getComponent();

				OutputWindow.this.setMinimumSize(component);
				OutputWindow.this.setMaximumSize(component);
			}
		});
	}

	private void setMinimumSize(Component component) {
		int width =
			component.getWidth() < WINDOW_WIDTH
				? WINDOW_WIDTH
				: component.getWidth();
		int height =
			component.getHeight() < WINDOW_HEIGHT
				? WINDOW_HEIGHT
				: component.getHeight();
		component.setSize(width, height);
	}

	private void setMaximumSize(Component component) {
		int width =
			component.getWidth() > WINDOW_WIDTH_MAX
				? WINDOW_WIDTH_MAX
				: component.getWidth();
		int height =
			component.getHeight() > WINDOW_HEIGHT_MAX
				? WINDOW_HEIGHT_MAX
				: component.getHeight();
		component.setSize(width, height);
	}

	private void calculateSize() {
		if (outputPanel != null) {
			int width = outputPanel.getPreferredSize().width + 30;
			int height = outputPanel.getPreferredSize().height + 90;
			this.setSize(width, height);
			setMinimumSize(this);
			setMaximumSize(this);
		}
	}

	public void close() {
		this.dispose();
	}
}
