/*
 * Created on 29.12.2004
 */
package ch.jaros.labyrinth.graphic;

import java.awt.Color;
import java.awt.Graphics;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import ch.jaros.labyrinth.Labyrinth;
import ch.jaros.labyrinth.Point;
import ch.jaros.labyrinth.Position4;

/**
 * @author Martin
 */
public abstract class AbstractLabyrinthGraphic implements LabyrinthGraphic {

	private Labyrinth labyrinth;

	public AbstractLabyrinthGraphic(Labyrinth labyrinth) {
		this.labyrinth = labyrinth;
	}

	public void draw(
		Graphics graphics,
		double factor,
		double imageOffsetX,
		double imageOffsetY) {
		if (labyrinth != null) {
			graphics.setColor(Color.BLACK);

			for (int y = 1; y < (labyrinth.getHeight() + 1); y++) {
				for (int x = 1; x < (labyrinth.getWidth() + 1); x++) {
					Point point = labyrinth.getPoint(new Position4(x, y));
					drawPoint(
						point,
						graphics,
						factor,
						imageOffsetX,
						imageOffsetY);
				}
			}
		}
	}

	abstract protected void drawPoint(
		Point point,
		Graphics graphics,
		double factor,
		double imageOffsetX,
		double imageOffsetY);


	protected int round(double d) {
		return (int) Math.round(d);
	}

	protected List getDirections(Point point) {
		List directions = new ArrayList();

		List connectedPoints = point.getConnections();
		Iterator it = connectedPoints.iterator();

		while (it.hasNext()) {
			Point connectedPoint = (Point) it.next();
			Integer direction =
				point.getPosition().directionTo(connectedPoint.getPosition());

			directions.add(direction);
		}

		return directions;
	}
}
