/*
 * Created on Feb 15, 2004
 */
package ch.jaros.labyrinth.graphic;

import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Image;
import java.awt.print.PageFormat;
import java.awt.print.Printable;
import java.awt.print.PrinterException;

import javax.swing.JComponent;

import ch.jaros.labyrinth.Labyrinth;
import ch.jaros.labyrinth.Labyrinth4;
import ch.jaros.labyrinth.Labyrinth6;
import ch.jaros.labyrinth.Labyrinth8;

/**
 * @author Martin Jaros
 */
public class LabyrinthComponent extends JComponent implements Printable {
	private int cellSize = 10;

	private Labyrinth labyrinth;

	private Image imgBuffer;

	public LabyrinthComponent() {
	}

	public void setLabyrinth(Labyrinth labyrinth) {
		this.labyrinth = labyrinth;

		calculatePreferredSize();

		imgBuffer = null;
	}

	public Labyrinth getLabyrinth() {
		return labyrinth;
	}

	public int getCellSize() {
		return cellSize;
	}

	public void setCellSize(int size) {
		this.cellSize = size;
		calculatePreferredSize();
		revalidate();
		repaint();
	}

	protected void paintComponent(Graphics g) {
		if (imgBuffer == null && labyrinth != null) {
			calculatePreferredSize();

			LabyrinthGraphic labyGraphic = getLabyrinthGraphic();

			Dimension dimension = getPreferredSize();
			imgBuffer = this.createImage(dimension.width, dimension.height);
			// drawing laby in the background
			labyGraphic.draw(imgBuffer.getGraphics(), cellSize, 0, 0);
		}

		g.drawImage(imgBuffer, 0, 0, null); // painting the cached labyrinth
	}

	private LabyrinthGraphic getLabyrinthGraphic() {
		LabyrinthGraphic labyGraphic = null;

		if (labyrinth instanceof Labyrinth4) {
			labyGraphic = new LabyrinthGraphic4(labyrinth);
		} else if (labyrinth instanceof Labyrinth6) {
			labyGraphic = new LabyrinthGraphic6(labyrinth);
		} else if (labyrinth instanceof Labyrinth8) {
			labyGraphic = new LabyrinthGraphic8(labyrinth);
		}

		return labyGraphic;
	}

	public void revalidate() {
		super.revalidate();
		imgBuffer = null;
	}

	public int print(Graphics g, PageFormat pf, int pi)
		throws PrinterException {
		if (pi >= 1) {
			return Printable.NO_SUCH_PAGE;
		}

		double pageHeightFactor =
			pf.getImageableHeight()
				/ ((labyrinth.getHeight() + 2) * getFactorX());
		double pageWidthFactor =
			pf.getImageableWidth()
				/ ((labyrinth.getWidth() + 2) * getFactorY());

		getLabyrinthGraphic().draw(
			g,
			Math.min(pageHeightFactor, pageWidthFactor),
			pf.getImageableX(),
			pf.getImageableY());

		return Printable.PAGE_EXISTS;
	}

	private void calculatePreferredSize() {
		if (labyrinth != null) {
			Dimension dimension =
				new Dimension(
					round((labyrinth.getWidth() + 2) * cellSize * getFactorX()),
					round(
						(labyrinth.getHeight() + 2) * cellSize * getFactorY()));

			setPreferredSize(dimension);
		}
	}

	private double getFactorX() {
		double factor = 1.0;

		if (labyrinth instanceof Labyrinth4) {
			factor = 1.0;
		} else if (labyrinth instanceof Labyrinth6) {
			factor = 1.55;
		} else if (labyrinth instanceof Labyrinth8) {
			factor = 1.0 + (1.0 / Math.sqrt(2.0));
		}

		return factor;
	}

	private double getFactorY() {
		double factor = 1.0;

		if (labyrinth instanceof Labyrinth4) {
			factor = 1.0;
		} else if (labyrinth instanceof Labyrinth6) {
			factor = 1.8;
		} else if (labyrinth instanceof Labyrinth8) {
			factor = 1.0 + (1.0 / Math.sqrt(2.0));
		}

		return factor;
	}

	private int round(double d) {
		return (int) Math.round(d);
	}
}
